# -*- coding: utf-8 -*-
try:
    from urlparse import urlparse, parse_qs
except ImportError:
    from urllib.parse import urlparse, parse_qs

from .baseapi import BaseAPI
from .baseapi import GET

class Manager(BaseAPI):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_experiment(self):
        """
            Create an experiment
        """
        return self.post_data("experiments", params={})

    def create_item(self, id):
        """
            Create an item, the id is the items_types id
        """
        return self.post_data("items/" + str(id), params={})

    def get_all_experiments(self):
        """
            This function returns a list of all experiments.
        """
        return self.get_data("experiments/")

    def get_experiment(self, id):
        """
            This function returns data from an experiment.
        """
        return self.get_data("experiments/" + str(id))

    def get_all_items(self):
        """
            Return all items
        """
        return self.get_data("items/")

    def get_item(self, id):
        """
            Get data from an item
        """
        return self.get_data("items/" + str(id))

    def get_items_types(self):
        """
            Get list of items_types
        """
        return self.get_data("items_types/")

    def get_status(self):
        """
            Get list of status
        """
        return self.get_data("status/")

    def post_experiment(self, id, params):
        """
            Change an experiment title/body/date
        """
        return self.post_data("experiments/" + str(id), params)

    def post_item(self, id, params):
        """
            Change an item title/body/date
            params must contain title, date and body
        """
        return self.post_data("items/" + str(id), params)

    def add_link_to_experiment(self, id, params):
        """
            Add a link to an experiment
            params must contain "link=<ITEM_ID>"
        """
        return self.post_data("experiments/" + str(id), params)

    def upload_to_experiment(self, id, params):
        """
            Upload a file to an experiment
        """
        return self.post_file("experiments/" + str(id), params)

    def upload_to_item(self, id, params):
        """
            Upload a file to an item
        """
        return self.post_file("items/" + str(id), params)

    def add_tag_to_experiment(self, id, params):
        """
            Add a tag to an experiment
        """
        return self.post_data("experiments/" + str(id), params)

    def add_tag_to_item(self, id, params):
        """
            Add a tag to an item
        """
        return self.post_data("items/" + str(id), params)

    def __str__(self):
        return "<Manager>"



# -*- coding: utf-8 -*-
"""A python module to interact with elabftw API"""

__version__ = "0.5.0"
__author__ = "Nicolas CARPi"
__author_email__ = "nicolas.carpi@curie.fr"
__license__ = "GPL v3"
__copyright__ = "Copyright (©) 2017 Nicolas CARPi"

from .Manager import Manager
from .baseapi import Error, SetupError, DataReadError



# -*- coding: utf-8 -*-
import os
import json
import logging
import requests
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

GET = 'GET'
POST = 'POST'
DELETE = 'DELETE'
PUT = 'PUT'

ACCESS_FORBIDDEN_MSG = "You don't have permission to access this resource!"

class Error(Exception):
    """Base exception class for this module"""
    pass

class SetupError(Error):
    pass

class DataReadError(Error):
    pass

class JSONReadError(Error):
    pass


class NotFoundError(Error):
    pass

class BaseAPI(object):
    """
        Basic api class for elabapy
    """
    token = ""
    endpoint = ""
    dev = False

    def __init__(self, *args, **kwargs):
        self.token = ""
        self.endpoint = ""
        self._log = logging.getLogger(__name__)
        self.dev = False

        for attr in kwargs.keys():
            setattr(self, attr, kwargs[attr])

    def __perform_request(self, url, type=GET, params=None):
        """
            This method will perform the real request,
            in this way we can customize only the "output" of the API call by
            using self.__call_api method.
            This method will return the request object.
        """
        if params is None:
            params = {}

        # check we have a token
        if not self.token:
            raise SetupError("No token provided.")

        # check we have an endpoint
        if not self.endpoint:
            raise SetupError("No endpoint provided.")

        # don't show warning about self-signed certificate in dev mode
        if self.dev:
            requests.packages.urllib3.disable_warnings()

        url = urljoin(self.endpoint, url)

        # lookup table to find out the apropriate requests method,
        # headers and payload type (json or query parameters)
        identity = lambda x: x
        json_dumps = lambda x: json.dumps(x)
        lookup = {
            GET: (requests.get, {}, 'params', identity),
            POST: (requests.post, {'Content-type': 'application/json'}, 'data',
                   json_dumps),
            PUT: (requests.put, {'Content-type': 'application/json'}, 'data',
                  json_dumps),
            DELETE: (requests.delete,
                     {'content-type': 'application/json'},
                     'data', json_dumps),
        }

        requests_method, headers, payload, transform = lookup[type]
        headers.update({'Authorization': self.token})
        kwargs = {'headers': headers, payload: transform(params)}

        # remove token from log
        headers_str = str(headers).replace(self.token.strip(), 'TOKEN')
        self._log.debug('%s %s %s:%s %s' %
                        (type, url, payload, params, headers_str))

        return requests_method(url, verify=False, **kwargs)

    def get_data(self, url, type=GET, params=None):
        """
            This method is a basic implementation of __call_api that checks
            errors too. In cas of success the method will return True or the
            content of the response to the request.
        """
        if params is None:
            params = dict()

        req = self.__perform_request(url, type, params)
        if req.status_code == 204:
            return True

        if req.status_code == 404:
            raise NotFoundError()

        if req.status_code == 403:
            return ACCESS_FORBIDDEN_MSG

        try:
            data = req.json()
        except ValueError as e:
            raise JSONReadError(
                'Read failed from API: %s' % str(e)
            )

        if not req.ok:
            msg = [data[m] for m in ("id", "message") if m in data][1]
            raise DataReadError(msg)

        return data

    def post_data(self, url, params):
        """
            POST some stuff to change title/date/body or create experiment
        """
        url = urljoin(self.endpoint, url)
        headers = {'Authorization': self.token}
        req = requests.post(url, headers=headers, data=params, verify=False)

        if req.status_code == 204:
            return True

        if req.status_code == 404:
            raise NotFoundError()

        if req.status_code == 403:
            return ACCESS_FORBIDDEN_MSG

        try:
            data = req.json()
        except ValueError as e:
            raise JSONReadError(
                'Read failed from API: %s' % str(e)
            )

        if not req.ok:
            msg = [data[m] for m in ("id", "message") if m in data][1]
            raise DataReadError(msg)

        return data

    def post_file(self, url, params):
        """
            POST files
        """
        url = urljoin(self.endpoint, url)
        headers = {'Authorization': self.token}
        req = requests.post(url, headers=headers, files=params, verify=False)

        if req.status_code == 204:
            return True

        if req.status_code == 404:
            raise NotFoundError()

        try:
            data = req.json()
        except ValueError as e:
            raise JSONReadError(
                'Read failed from API: %s' % str(e)
            )

        if not req.ok:
            msg = [data[m] for m in ("id", "message") if m in data][1]
            raise DataReadError(msg)

        return data

    def __str__(self):
        return "<%s>" % self.__class__.__name__

    def __unicode__(self):
        return u"%s" % self.__str__()

    def __repr__(self):
        return str(self)



#!/usr/bin/env python
import os
try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

long_description = """This library provides easy access to eLabFTW's API."""

setup(
    name='elabapy',
    version='0.5.0',
    description='elabftw API to manage experiments and items',
    author='Nicolas CARPi',
    author_email='nicolas.carpi@curie.fr',
    url='https://github.com/elabftw/elabapy',
    packages=['elabapy'],
    install_requires=['requests'],
    test_suite='elabapy.tests',
    license='GPL v3',
    long_description=long_description
)
